// Copyright (c) 2015, Electric Cloud, Inc.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//   - Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
//
//   - Redistributions in binary form must reproduce the above copyright
//     notice, this list of conditions and the following disclaimer in the
//     documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// (As an historical note, the above license instantiates
// the "BSD 2-Clause License" template that was presented
// by the Open Source Initiative at
//
//   http://www.opensource.org/licenses/bsd-license.php
//
// on 2012-02-23.)

// Summary:
//
// This header defines C++ wrappers for interfaces in:
//
//     "../interface/vrpsx_identified.h"
//
// NOTE: This convenience header does not affect the
// definition of any any binary interface; see README.txt.

#ifndef VRPSX_IDENTIFIED_HPP
#define VRPSX_IDENTIFIED_HPP 1

#include "../interface/vrpsx_identified.h"
#include "vrpsx_reference_counted.hpp"


namespace vrpsx {
    VRPSX_CPP_WRAPPER(identified_t);

    class identified_t
    {
    public:
        VRPSX_CONSTANT_METHOD(
            vrpsx_interface_identifier_t, interface_identifier)

        // TO DO: Figure out if there is any reasonable way to wrap the
        // "replace_interface" operation.  The wrapper would have to take
        // into account the kinds of smart pointer required for the old
        // and new interface types.  Those kinds might even differ!
    };


    VRPSX_CPP_WRAPPER(counted_identified_t);

    class counted_identified_t : public identified_t
    {
    public:
        void release()
        {
            vrpsx_counted_identified_t *iface = from_cpp(this);
            iface->vtbl->release(&iface);
        }

        void acquire()
        {
            vrpsx_counted_identified_t *iface = from_cpp(this);
            iface->vtbl->acquire(iface);
        }
    };

    inline void intrusive_ptr_add_ref(counted_identified_t *wrapper)
    {
        wrapper->acquire();
    }

    inline void intrusive_ptr_release(counted_identified_t *wrapper)
    {
        wrapper->release();
    }


    VRPSX_CPP_WRAPPER(error_t);

    class error_t : public counted_identified_t
    {
    public:
        char const *text() const
        {
            vrpsx_error_t const *iface = from_cpp(this);
            return iface->vtbl->text(iface);
        }
    };
}


#endif /* VRPSX_IDENTIFIED_HPP */
